import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleBand, scaleLinear, max } from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/curran/0ac4077c7fc6390f5dd33bf5c06cb5ff/raw/605c54080c7a93a417a3cea93fd52e7550e76500/UN_Population_2019.csv';
const width = 960;
const height = 500;
const margin = { top: 20, right: 20, bottom: 20, left: 220 };
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = (d) => {
      d.Population = +d['2020'];
      return d;
    };

    csv(csvUrl, row).then((data) => {
      setData(data.slice(0, 10));
    });
  }, []);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleLinear()
    .domain([0, max(data, (d) => d.Population)])
    .range([0, innerWidth]);

  const yScale = scaleBand()
    .domain(data.map((d) => d.Country))
    .range([0, innerHeight]);

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left}, ${margin.top})`}>
        {xScale.ticks().map((t) => (
          <g key={t} transform={`translate(${xScale(t)},0)`}>
            <line y2={innerHeight} stroke={'#ddd'} />
            <text
              y={innerHeight + 5}
              style={{ textAnchor: 'middle' }}
              dy="0.71em"
            >
              {t}
            </text>
          </g>
        ))}

        {yScale.domain().map((t) => (
          <text
            key={t}
            x={-3}
            transform={`translate(0,${yScale(t) + yScale.bandwidth() / 2})`}
            style={{ textAnchor: 'end' }}
            dy={'0.32em'}
          >
            {t}
          </text>
        ))}

        {data.map((d) => (
          <rect
            key={d.Country}
            x={0}
            y={yScale(d.Country)}
            width={xScale(d.Population)}
            height={yScale.bandwidth()}
          />
        ))}
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
